@@ -29,10 +29,12 @@ const GtkClutter = imports.gi.GtkClutter;
 const Lang = imports.lang;
 
 const CheckIn = imports.checkIn;
+const ContactPlace = imports.contactPlace;
 const Format = imports.format;
 const Geoclue = imports.geoclue;
 const GeocodeService = imports.geocodeService;
 const MainWindow = imports.mainWindow;
+const Maps = imports.gi.GnomeMaps;
 const NotificationManager = imports.notificationManager;
 const PlaceStore = imports.placeStore;
 const RouteService = imports.routeService;
@@ -49,6 +51,7 @@ let geoclue = null;
 let geocodeService = null;
 let networkMonitor = null;
 let checkInManager = null;
+let contactStore = null;
 
 const Application = new Lang.Class({
     Name: 'Application',
@@ -93,10 +96,50 @@ const Application = new Lang.Class({
         }).bind(this));
     },
 
+    _showContact: function(id) {
+        contactStore.lookup(id, (function(contact) {
+            this._mainWindow.markBusy();
+            contact.geocode((function() {
+                this._mainWindow.unmarkBusy();
+                this._mainWindow.mapView.showContact(contact);
+            }).bind(this));
+        }).bind(this));
+    },
+
+    _onShowContactActivate: function(action, parameter) {
+        this._createWindow();
+        this._checkNetwork();
+        this._mainWindow.window.present();
+
+        let id = parameter.deep_unpack();
+
+        if (contactStore.state === Maps.ContactStoreState.LOADED) {
+            this. _showContact(id);
+        } else {
+            Utils.once(contactStore, 'notify::state', (function() {
+                if (contactStore.state === Maps.ContactStoreState.LOADED)
+                    this._showContact(id);
+            }).bind(this));
+        }
+    },
+
     _onQuitActivate: function() {
         this._mainWindow.window.destroy();
     },
 
+    _addContacts: function() {
+        contactStore.get_contacts().forEach(function(contact) {
+            contact.geocode(function() {
+                contact.get_places().forEach(function(p) {
+                    Utils.debug('Adding contact address: ' + p.name);
+                    let place = new ContactPlace.ContactPlace({ place: p,
+                                                                contact: contact });
+                    placeStore.addPlace(place, PlaceStore.PlaceType.CONTACT);
+                });
+            });
+        });
+    },
+
     _initPlaceStore: function() {
         placeStore = new PlaceStore.PlaceStore();
         try {
@@ -105,6 +148,15 @@ const Application = new Lang.Class({
             log('Failed to parse Maps places file, ' +
                 'subsequent writes will overwrite the file!');
         }
+
+        if (contactStore.state === Maps.ContactStoreState.LOADED) {
+            this. _addContacts();
+        } else {
+            Utils.once(contactStore, 'notify::state', (function() {
+                if (contactStore.state === Maps.ContactStoreState.LOADED)
+                    this._addContacts();
+            }).bind(this));
+        }
     },
 
     _initAppMenu: function() {
@@ -126,7 +178,11 @@ const Application = new Lang.Class({
         this._initServices();
 
         Utils.addActions(this, {
-            'quit': { onActivate: this._onQuitActivate.bind(this) }
+            'quit': { onActivate: this._onQuitActivate.bind(this) },
+            'show-contact': {
+                paramType: 's',
+                onActivate: this._onShowContactActivate.bind(this)
+            }
         });
 
         this._initPlaceStore();
@@ -142,6 +198,8 @@ const Application = new Lang.Class({
         networkMonitor.connect('network-changed',
                                this._checkNetwork.bind(this));
         checkInManager = new CheckIn.CheckInManager();
+        contactStore = new Maps.ContactStore();
+        contactStore.load();
     },
 
     _createWindow: function() {
