@@ -29,9 +29,12 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 
 const Application = imports.application;
+const BusyMarker = imports.busyMarker;
 const ContextMenu = imports.contextMenu;
 const FavoritesPopover = imports.favoritesPopover;
+const Geoclue = imports.geoclue;
 const LayersPopover = imports.layersPopover;
+const LocationServiceNotification = imports.locationServiceNotification;
 const MapView = imports.mapView;
 const PlaceEntry = imports.placeEntry;
 const PlaceStore = imports.placeStore;
@@ -82,12 +85,16 @@ const MainWindow = new Lang.Class({
         this._toggleSidebarButton = ui.toggleSidebarButton;
         this._layersButton = ui.layersButton;
         this._favoritesButton = ui.favoritesButton;
+        this._busy = new BusyMarker.BusyMarker();
+        this._overlay.add_overlay(this._busy);
 
         this._initHeaderbar();
         this._initActions();
         this._initSignals();
         this._restoreWindowGeometry();
 
+        this._busySignalId = 0;
+
         ui.grid.attach(this._sidebar, 1, 0, 1, 1);
 
         ui.grid.show_all();
@@ -121,7 +128,7 @@ const MainWindow = new Lang.Class({
                                                this._setRevealSidebar.bind(this, true));
         sidebar.bind_property('reveal-child',
                               this.mapView, 'routeVisible',
-                              GObject.BindingFlags.DEFAULT);
+                              GObject.BindingFlags.BIDIRECTIONAL);
         this.window.application.bind_property('connected',
                                               sidebar, 'visible',
                                               GObject.BindingFlags.DEFAULT);
@@ -185,6 +192,13 @@ const MainWindow = new Lang.Class({
         }).bind(this));
     },
 
+    _updateLocationSensitivity: function() {
+        let sensitive = (Application.geoclue.state !== Geoclue.State.INITIAL &&
+                         this.window.application.connected);
+
+        this._gotoUserLocationButton.sensitive = sensitive;
+    },
+
     _initHeaderbar: function() {
         this._placeEntry = this._createPlaceEntry();
         this._headerBar.custom_title = this._placeEntry;
@@ -196,15 +210,12 @@ const MainWindow = new Lang.Class({
             this._favoritesButton.sensitive = favoritesPopover.rows > 0;
         }).bind(this));
 
-        Application.geoclue.connect('notify::connected', (function() {
-            this._gotoUserLocationButton.sensitive = Application.geoclue.connected;
-        }).bind(this));
-
+        Application.geoclue.connect('notify::state',
+                                    this._updateLocationSensitivity.bind(this));
         this.window.application.connect('notify::connected', (function() {
             let app = this.window.application;
 
-            this._gotoUserLocationButton.sensitive = (app.connected &&
-                                                      Application.geoclue.connected);
+            this._updateLocationSensitivity();
             this._layersButton.sensitive = app.connected;
             this._toggleSidebarButton.sensitive = app.connected;
             this._favoritesButton.sensitive = (app.connected &&
@@ -283,8 +294,37 @@ const MainWindow = new Lang.Class({
         return false;
     },
 
+    _getLocationServiceNotification: function() {
+        if (!this._locationServiceNotification) {
+            this._locationServiceNotification =
+                new LocationServiceNotification.LocationServiceNotification();
+        }
+
+        return this._locationServiceNotification;
+    },
+
     _onGotoUserLocationActivate: function() {
-        this.mapView.gotoUserLocation(true);
+        let message;
+        switch(Application.geoclue.state) {
+        case Geoclue.State.FAILED:
+            message = _("Failed to connect to location service");
+            Application.notificationManager.showMessage(message);
+            break;
+
+        case Geoclue.State.TIMEOUT:
+            message = _("Position not found");
+            Application.notificationManager.showMessage(message);
+            break;
+
+        case Geoclue.State.OFF:
+            let notification = this._getLocationServiceNotification();
+            Application.notificationManager.showNotification(notification);
+            break;
+
+        default:
+            this.mapView.gotoUserLocation(true);
+            break;
+        }
     },
 
     _onMapTypeMenuActivate: function(action) {
@@ -336,5 +376,25 @@ const MainWindow = new Lang.Class({
         aboutDialog.show();
         aboutDialog.connect('response',
                             aboutDialog.destroy.bind(aboutDialog));
+    },
+
+    markBusy: function() {
+        if (this._busySignalId !== 0)
+            return;
+
+        this._busy.show();
+
+        let stage = this.mapView.view.get_stage();
+        this._busySignalId = stage.connect('captured-event', function() {
+            return true;
+        });
+    },
+
+    unmarkBusy: function() {
+        this._busy.hide();
+
+        let stage = this.mapView.view.get_stage();
+        stage.disconnect(this._busySignalId);
+        this._busySignalId = 0;
     }
 });
