@@ -39,6 +39,33 @@ const Transportation = {
     }
 };
 
+const QueryPoint = new Lang.Class({
+    Name: 'QueryPoint',
+    Extends: GObject.Object,
+    Properties: {
+        'place': GObject.ParamSpec.object('place',
+                                          '',
+                                          '',
+                                          GObject.ParamFlags.READABLE |
+                                          GObject.ParamFlags.WRITABLE,
+                                          Geocode.Place)
+    },
+
+    _init: function() {
+        this._place = null;
+        this.parent();
+    },
+
+    set place(p) {
+        this._place = p;
+        this.notify('place');
+    },
+
+    get place() {
+        return this._place;
+    }
+});
+
 const RouteQuery = new Lang.Class({
     Name: 'RouteQuery',
     Extends: GObject.Object,
@@ -49,11 +76,11 @@ const RouteQuery = new Lang.Class({
     },
     Properties: {
         'points': GObject.ParamSpec.object('points',
-                                            '',
-                                            '',
+                                           '',
+                                           '',
                                            GObject.ParamFlags.READABLE |
                                            GObject.ParamFlags.WRITABLE,
-                                            GObject.Object),
+                                           GObject.Object),
         'transportation': GObject.ParamSpec.int('transportation',
                                                 '',
                                                 '',
@@ -141,30 +168,3 @@ const RouteQuery = new Lang.Class({
                "\nTransportation: " + this.transportation;
     }
 });
-
-const QueryPoint = new Lang.Class({
-    Name: 'QueryPoint',
-    Extends: GObject.Object,
-    Properties: {
-        'place': GObject.ParamSpec.object('place',
-                                          '',
-                                          '',
-                                          GObject.ParamFlags.READABLE |
-                                          GObject.ParamFlags.WRITABLE,
-                                          Geocode.Place),
-    },
-
-    _init: function() {
-        this._place = null;
-        this.parent();
-    },
-
-    set place(p) {
-        this._place = p;
-        this.notify('place');
-    },
-
-    get place() {
-        return this._place;
-    }
-});
